/***************************************************************************
 *   Copyright (C) 2010 by                                                 *
 *                                                                         *
 *   catWorkX GmbH, Schellerdamm 16, 21079 Hamburg                         *
 *   Sebastian Gerdes <sebastian.gerdes_AT_catworkx.de>                    *
 *                                                                         *
 *   This code is part of the Jira2Zarafa Plugin, which is able to add an  *
 *   all-day appointment in Zarafa. This all-day appointment will be added * 
 *   when you set a due date and an assignee in a Jira issue.              *
 *                                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Affero General Public License as        *
 *   published by the Free Software Foundation; either version 3           *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          *
 *   GNU AFFERO GENERAL PUBLIC LICENSE for more details.                   *
 *                                                                         *
 *   You should have received a copy of the GNU Affero General Public      *
 *   License along with this program.  If not, see                         *
 *   <http://www.gnu.org/licenses/>                                        *
 *                                                                         *
 ***************************************************************************/
 
package de.catworkx.jira.plugin.zarafa;

import com.atlassian.jira.ComponentManager;
import com.atlassian.jira.event.JiraEvent;
import com.atlassian.jira.event.issue.AbstractIssueEventListener;
import com.atlassian.jira.event.issue.IssueEvent;
import com.atlassian.jira.event.user.UserEvent;

import org.ofbiz.core.entity.GenericEntityException;
import org.ofbiz.core.entity.GenericValue;

import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * A listener for adding appointments to Zarafa when a due date and assignee is given.
 * 
 * @author Sebastian Gerdes <sebastian.gerdes@catworkx.de>
 * @version 1.0, 10/01/27
 * 
 */

public class Issue2Zarafa extends AbstractIssueEventListener {
	
	static String PHP_SOAP_URL = "PHP_SOAP_URL";
	static String[] acceptedParams = {PHP_SOAP_URL};

	
	//String phpSoapUrl = "	";
	MapiConnect mapi = null;
	
    public void init(Map params) {
    	// inititialize webservice
    	if (params.get(PHP_SOAP_URL) == null)
    		log("Error: No PHP_SOAP_URL given");
    	else {
    		String soapUrl = (String) params.get(PHP_SOAP_URL);
    		try {
    			this.mapi = new MapiConnect(soapUrl);
    		} catch (Exception e) {
    			log("Error: "+e.getMessage());
    		}
    	}
    }

    public String[] getAcceptedParams()
    {
        return this.acceptedParams;
    }

    // IssueEventListener implementation
    public void issueAssigned(IssueEvent event) {	

    	String assignee = event.getIssue().getAssigneeId();
    	String subject = event.getIssue().getSummary();
    	Timestamp duedate = event.getIssue().getDueDate();
    	String description = event.getIssue().getDescription();
    	Object oldAssignee = null;
    	// check if there has been an assignee before
		try {
			List changeItems;
			changeItems = event.getChangeLog().getRelated("ChildChangeItem");
		
			Iterator iter = changeItems.iterator();
			while (iter.hasNext()) {
				GenericValue changeItem = (GenericValue) iter.next();
				// log("allFields: " +changeItem.getAllFields());
				if (changeItem.get("field") == "assignee") {
							oldAssignee = changeItem.get("oldvalue");
				}
			}
		} catch (GenericEntityException e) {
			e.printStackTrace();
		}
		if (oldAssignee != null && duedate != null) {
			//log("found oldAssignee:" + oldAssignee);
			this.mapi.delAppointment(oldAssignee.toString(), subject, Long.toString(duedate.getTime()/1000));
		}
		else {
			log("NO oldAssignee found!");
		}
		if (assignee != null && duedate != null)
			this.mapi.setAppointment(assignee, subject, Long.toString(duedate.getTime()/1000), description);
    }


    public void issueCreated(IssueEvent event) {	
    	String assignee = event.getIssue().getAssigneeId();
    	String subject = event.getIssue().getSummary();
    	Timestamp duedate = event.getIssue().getDueDate();
    	String description = event.getIssue().getDescription();
 
    	if (assignee != null && duedate != null)
    		this.mapi.setAppointment(assignee, subject, Long.toString(duedate.getTime()/1000), description);
    }


    public void issueUpdated(IssueEvent event) {	
    	
    	String assignee = event.getIssue().getAssigneeId();
    	String subject = event.getIssue().getSummary();
    	Timestamp duedate = event.getIssue().getDueDate();
    	String description = event.getIssue().getDescription();
    	String oldSubject = subject;
    	Object oldAssignee = assignee;
    	String oldDuedate = duedate == null? null:Long.toString(duedate.getTime()/1000);
    	
    	// log("assignee: " + assignee);
    	// log("subject: " + subject);
    	// log("duedate: " + duedate.getTime()/1000);
    	// log("description: " + description);
    	
        try {
        	List changeItems = event.getChangeLog().getRelated("ChildChangeItem");
            Iterator iter = changeItems.iterator();
            while (iter.hasNext()) {
            	GenericValue changeItem = (GenericValue) iter.next();
            	// log("allFields: " +changeItem.getAllFields());
                if (changeItem.get("field") == "assignee")
                	oldAssignee = changeItem.get("oldvalue");
                if (changeItem.get("field") == "duedate") {
                	Object tmpdate = changeItem.get("oldstring");
                	if (tmpdate != null) {
                		String oldDuedateTmp = tmpdate.toString();
                		//2010-01-28 00:00:00.0
                		SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss.S");
                		Date parsedDate = dateFormat.parse(oldDuedateTmp);
                		oldDuedate = Long.toString(parsedDate.getTime()/1000);
                	}
                	
                }
                if (changeItem.get("field") == "summary")
                	oldSubject = changeItem.get("oldstring").toString();

                //log("\t"+changeItem.get("field") + " changed from '"+changeItem.get("oldstring")+"' to '"+changeItem.get("newstring") + "'");
            }
            // log("old assignee: " + oldAssignee);
            // log("old subject: " + oldSubject);
        	// log("old duedate: " + oldDuedate);
        	    
            // delete old appointment
        	if (oldAssignee != null && oldDuedate != null)
        		this.mapi.delAppointment(oldAssignee.toString(), oldSubject, oldDuedate);
            // add new appointment
            if (assignee != null && duedate != null)
            	this.mapi.setAppointment(assignee, subject, Long.toString(duedate.getTime()/1000), description);
        }
        catch (Exception e)
        {
            log("Error: "+e);
        }
    }

   

    public void issueDeleted(IssueEvent event) {
    	String assignee = event.getIssue().getAssigneeId();
    	String subject = event.getIssue().getSummary();
    	Timestamp duedate = event.getIssue().getDueDate();
    	//log("assignee: " + assignee);
    	//log("subject: " + subject);
    	//log("duedate: " + duedate.getTime()/1000);
    	if (assignee != null && duedate != null)
    		this.mapi.delAppointment(assignee, subject, Long.toString(duedate.getTime()/1000));
    	
    }


    private void logEvent(JiraEvent event)
    {
        try
        {
            if (event instanceof IssueEvent)
            {
                IssueEvent issueEvent = (IssueEvent) event;
                log("Issue: [#" + issueEvent.getIssue().getLong("id") + "] " + issueEvent.getIssue().getString("summary"));
                log("Comment: " + issueEvent.getComment());
                log("Change Group: " + issueEvent.getChangeLog());
                log("Event Type: " + ComponentManager.getInstance().getEventTypeManager().getEventType(issueEvent.getEventTypeId()).getName());
            }
            else if (event instanceof UserEvent)
            {
                UserEvent userEvent = (UserEvent) event;
                log("User: " + userEvent.getUser().getName() + " (" + userEvent.getUser().getEmail() + ")");
            }

            log(" Time: " + event.getTime());
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    private void log(String msg)
    {
        System.err.println("[DebugListener]: " + msg);
    }
}
